@@ -5,11 +5,11 @@ Main entry point: :func:`open_client`.
 """
 
 import anyio
-import outcome
 import socket
 import os
 from typing import Tuple
 from contextlib import asynccontextmanager, AsyncExitStack
+from inspect import iscoroutine
 
 from asyncscope import scope, Scope, main_scope
 
@@ -23,8 +23,10 @@ from .util import (
     combine_dict,
     ValueEvent,
     Path,
+    create_queue,
+    DelayedRead,
+    DelayedWrite,
 )
-from distmqtt.utils import create_queue
 from .default import CFG
 from .exceptions import (
     ClientAuthMethodError,
@@ -41,10 +43,7 @@ import logging
 
 logger = logging.getLogger(__name__)
 
-try:
-    ClosedResourceError = anyio.exceptions.ClosedResourceError
-except AttributeError:
-    ClosedResourceError = anyio.ClosedResourceError
+ClosedResourceError = anyio.ClosedResourceError
 
 rs = os.environ.get("PYTHONHASHSEED", None)
 if rs is None:
@@ -60,6 +59,22 @@ else:  # pragma: no cover
 __all__ = ["NoData", "ManyData", "open_client", "client_scope", "StreamedRequest"]
 
 
+class AsyncValueEvent(ValueEvent):
+    def set(self, value):
+        super().set(value)
+        return anyio.DeprecatedAwaitable(self.set)
+
+    def set_error(self, exc):
+        super().set_error(exc)
+        return anyio.DeprecatedAwaitable(self.set_error)
+
+    def cancel(self):
+        if self.scope is not None:
+            self.scope.cancel()
+        super().set_error(CancelledError())
+        return anyio.DeprecatedAwaitable(self.cancel)
+
+
 class NoData(ValueError):
     """No reply arrived"""
 
@@ -127,32 +142,46 @@ class StreamedRequest:
 
     start_msg = None
     end_msg = None
+    qr = None
+    dw = None
 
     def __init__(self, client, seq, stream: bool = False, report_start: bool = False):
         self._stream = stream
         self._client = client
         self.seq = seq
         self._stream = stream
-        self.q = create_queue(100)
         self._client._handlers[seq] = self
         self._reply_stream = None
         self.n_msg = 0
         self._report_start = report_start
-        self._started = anyio.create_event()
+        self._started = anyio.Event()
         self._path_long = lambda x: x
-        # None: no message yet; True: begin seen; False: end or single message seen
+        if client.qlen > 0:
+            self.dw = DelayedWrite(client.qlen)
+            self.qr = DelayedRead(client.qlen, get_seq=self._get_seq, send_ack=self._send_ack)
+        else:
+            self.qr = create_queue(client.config.server.buffer)
+
+    @staticmethod
+    def _get_seq(msg):
+        return msg.pop("wseq", 0)
+
+    async def _send_ack(self, seq):
+        await self._client._send(seq=self.seq, state="ack", ack=seq)
 
     async def set(self, msg):
         """Called by the read loop to process a command's result"""
         self.n_msg += 1
         if "error" in msg:
             logger.info("ErrorMsg: %s", msg)
-            if self.q is not None:
-                try:
-                    cls = error_types[msg["etype"]]
-                except KeyError:
-                    cls = ServerError
-                await self.q.put(outcome.Error(cls(msg.error)))
+            try:
+                cls = error_types[msg["etype"]]
+            except KeyError:
+                cls = ServerError
+            try:
+                await self.qr.put_error(cls(msg.error))
+            except anyio.BrokenResourceError:
+                raise cls(msg.error)
             return
         state = msg.get("state", "")
 
@@ -161,19 +190,23 @@ class StreamedRequest:
                 raise RuntimeError("Recv state 2", self._reply_stream, msg)
             self._reply_stream = True
             self.start_msg = msg
-            await self._started.set()
+            self._started.set()
             if self._report_start:
-                await self.q.put(outcome.Value(msg))
+                await self.qr.put(msg)
 
         elif state == "end":
             if self._reply_stream is not True:  # pragma: no cover
                 raise RuntimeError("Recv state 3", self._reply_stream, msg)
             self._reply_stream = None
             self.end_msg = msg
-            if self.q is not None:
-                await self.q.put(None)
+            if self.qr is not None:
+                await self.qr.close_sender()
             return False
 
+        elif state == "ack":
+            if self.dw is not None:
+                await self.dw.recv_ack(msg["ack"])
+
         else:
             if state not in ("", "uptodate"):  # pragma: no cover
                 logger.warning("Unknown state: %s", msg)
@@ -182,10 +215,13 @@ class StreamedRequest:
                 raise RuntimeError("Recv state 1", self._reply_stream, msg)
             elif self._reply_stream is None:
                 self._reply_stream = False
-            if self.q is not None:
-                await self.q.put(outcome.Value(msg))
-                if self._reply_stream is False:
-                    await self.q.put(None)
+            try:
+                with anyio.fail_after(1):
+                    await self.qr.put(msg)
+            except anyio.BrokenResourceError:
+                logger.warning("Reader for %s closed: %s", self.seq, msg)
+            if self._reply_stream is False:
+                await self.qr.close_sender()
 
     async def get(self):
         """Receive a single reply"""
@@ -206,12 +242,10 @@ class StreamedRequest:
         return self
 
     async def __anext__(self):
-        res = await self.q.get()
-        if res is None:
-            self.q = None  # prevent deadlock if called again
-            raise StopAsyncIteration
         try:
-            res = res.unwrap()
+            res = await self.qr.get()
+        except (anyio.EndOfStream, anyio.ClosedResourceError):
+            raise StopAsyncIteration
         except CancelledError:
             raise StopAsyncIteration  # just terminate
         self._path_long(res)
@@ -230,26 +264,31 @@ class StreamedRequest:
         elif self._stream == 2 and params.get("state", "") == "end":
             self._stream = None
         logger.debug("Send %s", {"seq": self.seq, **params})
+        if self.dw is not None:
+            params["wseq"] = await self.dw.next_seq()
         await self._client._send(seq=self.seq, **params)
 
     async def recv(self):
         return await self.__anext__()
 
     async def cancel(self):
-        if self.q is not None:
-            await self.q.put(outcome.Error(CancelledError()))
-        await self.aclose()
+        try:
+            await self.qr.put_error(CancelledError())
+        except (anyio.BrokenResourceError, anyio.ClosedResourceError):
+            pass
+        else:
+            await self.aclose()
 
     async def wait_started(self):
         await self._started.wait()
 
     async def aclose(self, timeout=0.2):
-        if self.q is not None:
-            await self.q.put(None)
+        await self.qr.close_sender()
+        await self.qr.close_receiver()
         if self._stream == 2:
             await self._client._send(seq=self.seq, state="end")
             if timeout is not None:
-                async with anyio.move_on_after(timeout):
+                with anyio.move_on_after(timeout):
                     try:
                         await self.recv()
                     except StopAsyncIteration:
@@ -268,7 +307,7 @@ class _SingleReply:
     def __init__(self, conn, seq, params):
         self._conn = conn
         self.seq = seq
-        self.q = ValueEvent()
+        self.q = AsyncValueEvent()
         self._params = params
 
     async def set(self, msg):
@@ -308,7 +347,7 @@ class ClientConfig:
     def _init(self, client):
         self._client = client
         self._current = {}
-        self._changed = anyio.create_event()
+        self._changed = anyio.Event()
 
     def __getattr__(self, k):
         if k.startswith("_"):
@@ -330,8 +369,8 @@ class ClientConfig:
         configuration; the old data is discarded.
         """
         self._current[k] = combine_dict(v, self._client._cfg.get(k, {}))
-        c, self._changed = self._changed, anyio.create_event()
-        await c.set()
+        c, self._changed = self._changed, anyio.Event()
+        c.set()
 
     async def _watch(self):
         class CfgWatcher:
@@ -362,6 +401,7 @@ class Client:
 
     server_name = None
     client_name = None
+    qlen: int = 0
 
     def __init__(self, cfg: dict):
         self._cfg = combine_dict(cfg, CFG, cls=attrdict)
@@ -369,7 +409,7 @@ class Client:
 
         self._seq = 0
         self._handlers = {}
-        self._send_lock = anyio.create_lock()
+        self._send_lock = anyio.Lock()
         self._helpers = {}
         self._name = "".join(random.choices("abcdefghjkmnopqrstuvwxyz23456789", k=9))
 
@@ -428,19 +468,11 @@ class Client:
                 k.generate_public_key()
                 return k
 
-            try:
-                k = await anyio.run_in_thread(gen_key)
-            except AttributeError:
-                k = await anyio.run_sync_in_worker_thread(gen_key)
+            k = await anyio.to_thread.run_sync(gen_key)
             res = await self._request(
                 "diffie_hellman", pubkey=num2byte(k.public_key), length=length
             )  # length=k.key_length
-            try:
-                await anyio.run_in_thread(k.generate_shared_secret, byte2num(res.pubkey))
-            except AttributeError:
-                await anyio.run_sync_in_worker_thread(
-                    k.generate_shared_secret, byte2num(res.pubkey)
-                )
+            await anyio.to_thread.run_sync(k.generate_shared_secret, byte2num(res.pubkey))
             self._dh_key = num2byte(k.shared_secret)[0:32]
         return self._dh_key
 
@@ -454,16 +486,13 @@ class Client:
                 p = packer(params)
             except TypeError as e:
                 raise ValueError(f"Unable to pack: {params!r}") from e
-            try:
-                await sock.send_all(p)
-            except AttributeError:
-                await sock.send(p)
+            await sock.send(p)
 
     async def _reader(self, *, evt=None):
         """Main loop for reading"""
         unpacker = stream_unpacker()
 
-        async with anyio.open_cancel_scope():
+        with anyio.CancelScope():
             # XXX store the scope so that the redaer may get cancelled?
             if evt is not None:
                 await evt.set()
@@ -489,11 +518,13 @@ class Client:
                     unpacker.feed(buf)
 
             finally:
-                async with anyio.fail_after(2, shield=True):
+                with anyio.fail_after(2, shield=True):
                     hdl, self._handlers = self._handlers, None
                     for m in hdl.values():
                         try:
-                            await m.cancel()
+                            res = m.cancel()
+                            if iscoroutine(res):
+                                await res
                         except ClosedResourceError:
                             pass
 
@@ -550,10 +581,13 @@ class Client:
 
         elif iter is False and isinstance(res, StreamedRequest):
             rr = None
-            async for r in res:
-                if rr is not None:
-                    raise ManyData(action)
-                rr = r
+            try:
+                async for r in res:
+                    if rr is not None:
+                        raise ManyData(action)
+                    rr = r
+            finally:
+                await res.aclose()
             if rr is None:
                 raise NoData(action)
             res = rr
@@ -611,7 +645,7 @@ class Client:
                 await res.send(error=repr(exc))
             raise
         finally:
-            async with anyio.fail_after(2, shield=True):
+            with anyio.fail_after(2, shield=True):
                 await res.aclose()
 
     async def _run_auth(self, auth=None):
@@ -639,7 +673,7 @@ class Client:
         This async context manager handles the actual TCP connection to
         the DistKV server.
         """
-        hello = ValueEvent()
+        hello = AsyncValueEvent()
         self._handlers[0] = hello
 
         cfg = self._cfg["connect"]
@@ -669,11 +703,14 @@ class Client:
                 try:
                     self._socket = stream
                     await self.scope.spawn(self._reader)
-                    async with anyio.fail_after(init_timeout):
-                        self._server_init = await hello.get()
-                        logger.debug("Hello %s", self._server_init)
-                        self.server_name = self._server_init.node
-                        self.client_name = cfg["name"] or self.server_name
+                    with anyio.fail_after(init_timeout):
+                        self._server_init = msg = await hello.get()
+                        logger.debug("Hello %s", msg)
+                        self.server_name = msg.node
+                        self.client_name = cfg["name"] or socket.gethostname() or self.server_name
+                        if "qlen" in msg:
+                            self.qlen = min(msg.qlen, self.config.server.buffer)
+                            await self._send(seq=0, qlen=self.qlen)
                         await self._run_auth(auth)
 
                     from .config import ConfigRoot
@@ -687,7 +724,7 @@ class Client:
                 else:
                     yield self
                 finally:
-                    async with anyio.fail_after(2, shield=True):
+                    with anyio.fail_after(2, shield=True):
                         # Clean up our hacked config
                         try:
                             del self._config
