@@ -6,8 +6,9 @@ import io
 import signal
 import time
 import anyio
+from anyio.abc import SocketAttribute
 
-from distmqtt.utils import create_queue
+from .util import create_queue, DelayedRead, DelayedWrite
 
 try:
     from contextlib import asynccontextmanager
@@ -36,7 +37,7 @@ from .util import (
     MsgReader,
     combine_dict,
     drop_dict,
-    create_tcp_server,
+    run_tcp_server,
     gen_ssl,
     num2byte,
     byte2num,
@@ -90,11 +91,28 @@ def cmp_n(a, b):
     return b - a
 
 
+class HelloProc:
+    """
+    A hacked-up command processor for receiving the first client message.
+    """
+
+    def __init__(self, client):
+        self.client = client
+
+    async def received(self, msg):
+        qlen = msg.get("qlen", 0)
+        self.client.qlen = min(qlen, self.client.server.cfg.server.buffer)
+        del self.client.in_stream[0]
+
+    async def aclose(self):
+        self.client.in_stream.pop(0, None)
+
+
 class StreamCommand:
     """Represent the execution of a streamed command.
 
     Implement the actual command by overriding ``run``.
-    Read the next input line by reading ``in_q``.
+    Read the next input message by calling ``recv``.
 
     This auto-detects whether the client sends multiple lines, by closing
     the incoming channel if there's no state=start in the command.
@@ -108,6 +126,8 @@ class StreamCommand:
     send_q = None
     _scope = None
     end_msg = None
+    qr = None
+    dw = None
 
     def __new__(cls, client, msg):
         if cls is StreamCommand:
@@ -120,42 +140,52 @@ class StreamCommand:
         self.client = client
         self.msg = msg
         self.seq = msg.seq
-        self.in_q = create_queue(1)
         self.client.in_stream[self.seq] = self
+        self.qlen = self.client.qlen
+        if self.qlen:
+            self.qr = DelayedRead(self.qlen, get_seq=self._get_seq, send_ack=self._send_ack)
+            self.dw = DelayedWrite(self.qlen)
+        else:
+            self.qr = create_queue(1)
+
+    @staticmethod
+    def _get_seq(msg):
+        return msg.get("wseq", 0)
+
+    async def _send_ack(self, seq):
+        await self.client.send(seq=self.seq, state="ack", ack=seq)
 
     async def received(self, msg):
         """Receive another message from the client"""
 
         s = msg.get("state", "")
+        if s == "ack":
+            if self.dw is not None:
+                await self.dw.recv_ack(msg["ack"])
+            return
+
         err = msg.get("error", None)
         if err:
-            await self.in_q.put(msg)
+            await self.qr.put(msg)
         if s == "end":
             self.end_msg = msg
             await self.aclose()
         elif not err:
-            await self.in_q.put(msg)
+            await self.qr.put(msg)
 
     async def aclose(self):
         self.client.in_stream.pop(self.seq, None)
-        if self.in_q is not None:
-            await self.in_q.put(None)
-            self.in_q = None
+        await self.qr.close_sender()
 
     async def recv(self):
-        msg = await self.in_q.get()
-        if msg is None:
-            raise ServerClosedError
+        msg = await self.qr.get()
 
         if "error" in msg:
             raise ClientError(msg.error)
         return msg
 
     async def send(self, **msg):
-        """Send a message to the client.
-
-        TODO add a rate limit
-        """
+        """Send a message to the client."""
         msg["seq"] = self.seq
         if not self.multiline:
             if self.multiline is None:
@@ -164,6 +194,8 @@ class StreamCommand:
         elif self.multiline == -1:
             raise RuntimeError("Can't explicitly send in simple interaction")
         try:
+            if self.dw is not None:
+                msg["wseq"] = await self.dw.next_seq()
             await self.client.send(msg)
         except ClosedResourceError:
             self.client.logger.info("OERR %d", self.client._client_nr)
@@ -172,10 +204,11 @@ class StreamCommand:
         msg = self.msg
         if msg.get("state") != "start":
             # single message
-            if self.in_q is not None:
-                await self.in_q.put(None)
-                self.in_q = None
+            await self.qr.close_sender()
 
+        qlen = msg.get("qlen", 0)
+        if qlen > 0:
+            self.dw = DelayedWrite(qlen)
         if self.multiline > 0:
             await self.send(state="start")
             try:
@@ -187,8 +220,11 @@ class StreamCommand:
                     self.client.logger.exception("ERS%d %r", self.client._client_nr, self.msg)
                 await self.send(error=repr(exc))
             finally:
-                async with anyio.move_on_after(2, shield=True):
-                    await self.send(state="end")
+                with anyio.move_on_after(2, shield=True):
+                    try:
+                        await self.send(state="end")
+                    except anyio.BrokenResourceError:
+                        pass
 
         else:
             res = await self.run(**kw)
@@ -531,7 +567,7 @@ class SCmd_watch(StreamCommand):
                         await entry.walk(worker, acl=acl, **kv)
                         await self.send(state="uptodate")
 
-                    await tg.spawn(orig_state)
+                    tg.spawn(orig_state)
 
                 async for m in watcher:
                     ml = len(m.entry.path) - len(msg.path)
@@ -605,6 +641,7 @@ class ServerClient:
     conv = ConvNull
     acl: ACLStepper = NullACL
     tg = None
+    qlen = 0
 
     def __init__(self, server: "Server", stream: Stream):
         self.server = server
@@ -612,9 +649,9 @@ class ServerClient:
         self.metaroot = self.root.follow(Path(None), create=True, nulls_ok=True)
         self.stream = stream
         self.tasks = {}
-        self.in_stream = {}
+        self.in_stream = {0: HelloProc(self)}
         self._chop_path = 0
-        self._send_lock = anyio.create_lock()
+        self._send_lock = anyio.Lock()
 
         global _client_nr
         _client_nr += 1
@@ -649,7 +686,7 @@ class ServerClient:
         self.logger.debug("IN_%d %s", self._client_nr, msg)
 
         seq = msg.seq
-        async with anyio.open_cancel_scope() as s:
+        with anyio.CancelScope() as s:
             self.tasks[seq] = s
             if "chain" in msg:
                 msg.chain = NodeEvent.deserialize(msg.chain, cache=self.server.node_cache)
@@ -666,7 +703,7 @@ class ServerClient:
                     raise NoAuthError()
                 fn = partial(self._process, fn, msg)
             if evt is not None:
-                await evt.set()
+                evt.set()
 
             try:
                 await fn()
@@ -705,10 +742,7 @@ class ServerClient:
             return k
 
         async with self.server.crypto_limiter:
-            try:
-                k = await anyio.run_in_thread(gen_key)
-            except AttributeError:
-                k = await anyio.run_sync_in_worker_thread(gen_key)
+            k = await anyio.to_thread.run_sync(gen_key)
         return {"pubkey": num2byte(k.public_key)}
 
     cmd_diffie_hellman.noAuth = True
@@ -1096,7 +1130,7 @@ class ServerClient:
             if "tock" not in msg:
                 msg["tock"] = self.server.tock
             try:
-                await self.stream.send_all(packer(msg))
+                await self.stream.send(packer(msg))
             except ClosedResourceError:
                 self.logger.info("ERO%d %r", self._client_nr, msg)
                 self._send_lock = None
@@ -1122,6 +1156,7 @@ class ServerClient:
                 "node": self.server.node.name,
                 "tick": self.server.node.tick,
                 "tock": self.server.tock,
+                "qlen": self.server.cfg.server.buffer,
             }
             try:
                 auth = self.root.follow(Path(None, "auth"), nulls_ok=True, create=False)
@@ -1155,8 +1190,8 @@ class ServerClient:
                         if send_q is not None:
                             await send_q.received(msg)
                         else:
-                            evt = anyio.create_event()
-                            await self.tg.spawn(self.process, msg, evt)
+                            evt = anyio.Event()
+                            self.tg.spawn(self.process, msg, evt)
                             await evt.wait()
                     except Exception as exc:
                         msg = {"error": str(exc)}
@@ -1172,7 +1207,7 @@ class ServerClient:
                         await self.send(msg)
 
                 try:
-                    buf = await self.stream.receive_some(4096)
+                    buf = await self.stream.receive(4096)
                 except ConnectionResetError:
                     self.logger.info("DEAD %d", self._client_nr)
                     break
@@ -1181,7 +1216,7 @@ class ServerClient:
                     break
                 unpacker_.feed(buf)
 
-            await tg.cancel_scope.cancel()
+            tg.cancel_scope.cancel()
 
     def drop_old_event(self, evt, old_evt=NotGiven):
         return self.server.drop_old_event(evt, old_evt)
@@ -1233,26 +1268,26 @@ class _RecoverControl:
         return id(self)
 
     async def cancel(self):
-        await self.scope.cancel()
+        self.scope.cancel()
         rt = self.server._recover_tasks
         for node in self.local_history:
             if rt.get(node, None) is self:
                 del rt[node]
         self.local_history = ()
         for evt in list(self._waiters.values()):
-            await evt.set()
+            evt.set()
 
     async def set(self, n):
         evt = self._waiters.get(n, None)
         if evt is None:
-            evt = anyio.create_event()
+            evt = anyio.Event()
             self._waiters[n] = evt
-        await evt.set()
+        evt.set()
 
     async def wait(self, n):
         evt = self._waiters.get(n, None)
         if evt is None:
-            evt = anyio.create_event()
+            evt = anyio.Event()
             self._waiters[n] = evt
         await evt.wait()
 
@@ -1304,12 +1339,12 @@ class Server:
         self.node = Node(name, None, cache=self.node_cache)
 
         self._init = init
-        self.crypto_limiter = anyio.create_semaphore(3)
+        self.crypto_limiter = anyio.Semaphore(3)
         self.logger = logging.getLogger("distkv.server." + name)
         self._delete_also_nodes = NodeSet()
 
         # Lock for generating a new node event
-        self._evt_lock = anyio.create_lock()
+        self._evt_lock = anyio.Lock()
 
         # connected clients
         self._clients = set()
@@ -1384,7 +1419,7 @@ class Server:
                 if n is not None:
                     self.logger.warning("Deletion %s %d due to %r", self.node, n.tick, exc)
                     self.node.report_deleted(RangeSet((nt,)), self)
-                    async with anyio.move_on_after(2, shield=True):
+                    with anyio.move_on_after(2, shield=True):
                         await self._send_event(
                             "info", dict(node="", tick=0, deleted={self.node.name: (nt,)})
                         )
@@ -1812,7 +1847,7 @@ class Server:
                     if not msg:  # None, empty, whatever
                         continue
                     self.logger.debug("Recv %s: %r", action, msg)
-                    async with anyio.fail_after(10):
+                    with anyio.fail_after(10):
                         await self.tock_seen(msg.get("tock", 0))
                         await cmd(msg)
         except CancelledError:
@@ -1852,12 +1887,12 @@ class Server:
         ) as actor:
             self._actor = actor
             await self._check_ticked()
-            await delay.set()
+            delay.set()
             async for msg in actor:
                 # self.logger.debug("IN %r",msg)
 
                 if isinstance(msg, RecoverEvent):
-                    await self.spawn(
+                    self.spawn(
                         self.recover_split,
                         msg.prio,
                         msg.replace,
@@ -1866,7 +1901,7 @@ class Server:
                     )
 
                 elif isinstance(msg, GoodNodeEvent):
-                    await self.spawn(self.fetch_data, msg.nodes)
+                    self.spawn(self.fetch_data, msg.nodes)
 
                 elif isinstance(msg, RawMsgEvent):
                     msg = msg.msg
@@ -1900,6 +1935,9 @@ class Server:
         While you can use the hostmap to temporarily add new hosts with
         unusual addresses, the new host still needs a config entry.
         """
+
+        # this is async because the test mock needs that
+
         port = self.cfg.connect.port
         domain = self.cfg.domain
         try:
@@ -2054,8 +2092,9 @@ class Server:
                         self.fetch_missing.add(nst)
                 if len(self.fetch_missing):
                     self.fetch_running = False
-                    self.logger.error("Sync: missing: %s", self.fetch_missing)
-                    await self.spawn(self.do_send_missing)
+                    for nm in self.fetch_missing:
+                        self.logger.error("Sync: missing: %s %s", nm.name, nm.local_missing)
+                    self.spawn(self.do_send_missing)
                 if self.force_startup or not len(self.fetch_missing):
                     if self.node.tick is None:
                         self.node.tick = 0
@@ -2122,7 +2161,7 @@ class Server:
             return
         if self.node.tick is not None:
             self.logger.debug("Ready")
-            await self._ready.set()
+            self._ready.set()
             await self._set_tock()
         else:
             # self.logger.debug("Not yet ready.")
@@ -2132,7 +2171,7 @@ class Server:
         """
         Recover from a network split.
         """
-        async with anyio.open_cancel_scope() as scope:
+        with anyio.CancelScope() as scope:
             for node in sources:
                 if node not in self._recover_tasks:
                     break
@@ -2153,7 +2192,7 @@ class Server:
 
                 # Step 1: send an info/ticks message
                 # for prio=0 this fires immediately. That's intentional.
-                async with anyio.move_on_after(clock * (1 - 1 / (1 << prio))) as x:
+                with anyio.move_on_after(clock * (1 - 1 / (1 << prio))) as x:
                     await t.wait(1)
                 if x.cancel_called:
                     msg = dict((x.name, x.tick) for x in self._nodes.values())
@@ -2168,7 +2207,7 @@ class Server:
                 # chance to wait for other info/ticks messages. We can't
                 # trigger on them because there may be more than one, for a
                 # n-way merge.
-                async with anyio.move_on_after(clock * (2 - 1 / (1 << prio)) / 2) as x:
+                with anyio.move_on_after(clock * (2 - 1 / (1 << prio)) / 2) as x:
                     await t.wait(2)
 
                 if x.cancel_called:
@@ -2183,7 +2222,7 @@ class Server:
                 await self._run_send_missing(prio)
 
             finally:
-                async with anyio.open_cancel_scope(shield=True):
+                with anyio.CancelScope(shield=True):
                     # Protect against cleaning up when another recovery task has
                     # been started (because we saw another merge)
                     self.logger.debug("SplitRecover %d: finished @%d", t._id, t.tock)
@@ -2218,7 +2257,7 @@ class Server:
 
         if self.sending_missing is None:
             self.sending_missing = True
-            await self.spawn(self._send_missing_data, prio)
+            self.spawn(self._send_missing_data, prio)
         elif not self.sending_missing:
             self.sending_missing = True
 
@@ -2410,7 +2449,7 @@ class Server:
                         cnt = 0
 
                     try:
-                        async with anyio.fail_after(1 if cnt else 60 - td):
+                        with anyio.fail_after(1 if cnt else 60 - td):
                             msg = await updates.__anext__()
                     except TimeoutError:
                         await mw.flush()
@@ -2430,8 +2469,8 @@ class Server:
         self, path: str = None, stream=None, done: ValueEvent = None, save_state=False
     ):
 
-        async with anyio.open_cancel_scope() as s:
-            sd = anyio.create_event()
+        with anyio.CancelScope() as s:
+            sd = anyio.Event()
             state = (s, sd)
             self._savers.append(state)
             try:
@@ -2441,9 +2480,9 @@ class Server:
             except EnvironmentError as err:
                 if done is None:
                     raise
-                await done.set_error(err)
+                done.set_error(err)
             finally:
-                async with anyio.open_cancel_scope(shield=True):
+                with anyio.CancelScope(shield=True):
                     await sd.set()
 
     async def run_saver(self, path: str = None, stream=None, save_state=False, wait: bool = True):
@@ -2466,7 +2505,7 @@ class Server:
         done = ValueEvent() if wait else None
         res = None
         if path is not None:
-            await self.spawn(
+            self.spawn(
                 partial(self._saver, path=path, stream=stream, save_state=save_state, done=done)
             )
             if wait:
@@ -2479,7 +2518,7 @@ class Server:
             await sd.wait()
 
     async def _sigterm(self):
-        async with anyio.open_signal_receiver(signal.SIGTERM) as r:
+        with anyio.open_signal_receiver(signal.SIGTERM) as r:
             async for s in r:
                 for s, sd in self._savers:
                     await s.cancel()
@@ -2498,7 +2537,7 @@ class Server:
         await self._ready2.wait()
 
     async def serve(self, log_path=None, log_inc=False, force=False, ready_evt=None):
-        """Task that opens a Serf connection and actually runs the server.
+        """Task that opens a backend connection and actually runs the server.
 
         Args:
           ``setup_done``: optional event that's set when the server is initially set up.
@@ -2533,10 +2572,10 @@ class Server:
 
             # Set when self.node.tick is no longer None, i.e. we have some
             # reasonable state
-            self._ready = anyio.create_event()
+            self._ready = anyio.Event()
 
             # set when we're ready to accept client connections
-            self._ready2 = anyio.create_event()
+            self._ready2 = anyio.Event()
 
             self.serf = serf
             self.spawn = serf.spawn
@@ -2554,12 +2593,12 @@ class Server:
 
             # used to sync starting up everything so no messages get either
             # lost, or processed prematurely
-            delay = anyio.create_event()
-            delay2 = anyio.create_event()
-            delay3 = anyio.create_event()
+            delay = anyio.Event()
+            delay2 = anyio.Event()
+            delay3 = anyio.Event()
 
-            await self.spawn(self._run_del, delay3)
-            await self.spawn(self._delete_also)
+            self.spawn(self._run_del, delay3)
+            self.spawn(self._delete_also)
 
             if log_path is not None:
                 await self.run_saver(path=log_path, save_state=not log_inc, wait=False)
@@ -2567,10 +2606,10 @@ class Server:
             # Link up our "user_*" code
             for d in dir(self):
                 if d.startswith("user_"):
-                    await self.spawn(self.monitor, d[5:], delay)
+                    self.spawn(self.monitor, d[5:], delay)
 
             await delay3.wait()
-            await self.spawn(self.watcher)
+            self.spawn(self.watcher)
 
             if self._init is not NotGiven:
                 assert self.node.tick is None
@@ -2578,13 +2617,13 @@ class Server:
                 async with self.next_event() as event:
                     await self.root.set_data(event, self._init, tock=self.tock, server=self)
 
-            await self.spawn(self._sigterm)
+            self.spawn(self._sigterm)
 
             # send initial ping
-            await self.spawn(self._pinger, delay2)
+            self.spawn(self._pinger, delay2)
 
             await anyio.sleep(0.1)
-            await delay.set()
+            delay.set()
             await self._check_ticked()  # when _init is set
             await delay2.wait()
             await self._ready.wait()
@@ -2595,33 +2634,30 @@ class Server:
             async with anyio.create_task_group() as tg:
                 for n, cfg in enumerate(cfgs):
                     cfg = combine_dict(cfg, cfg_b, cls=attrdict)
-                    evt = anyio.create_event()
+                    evt = anyio.Event()
                     evts.append(evt)
-                    await tg.spawn(self._accept_clients, cfg, n, evt)
+                    tg.spawn(self._accept_clients, tg, cfg, n, evt)
                 for evt in evts:
                     await evt.wait()
 
-                await self._ready2.set()
+                self._ready2.set()
                 if ready_evt is not None:
                     await ready_evt.set()
                 pass  # end of server taskgroup
             pass  # end of server
         pass  # end of serf client
 
-    async def _accept_clients(self, cfg, n, evt):
+    async def _accept_clients(self, tg, cfg, n, evt):
         ssl_ctx = gen_ssl(cfg["ssl"], server=True)
         cfg = combine_dict({"ssl": ssl_ctx}, cfg, cls=attrdict)
 
-        async with create_tcp_server(**cfg) as server:
+        def rdy(n, server):
             if n == 0:
-                self.ports = server.ports
-            if evt is not None:
-                await evt.set()
+                port = server.extra(SocketAttribute.local_address)
+            self.ports = [port]
+            evt.set()
 
-            async for stream in server:
-                await self.spawn(self._connect, stream)
-            pass  # unwinding create_tcp_server
-        pass  # unwinding the client (cancelled or error)
+        await run_tcp_server(self._connect, tg=tg, _rdy=partial(rdy, n), **cfg)
 
     async def _connect(self, stream):
         c = None
@@ -2629,7 +2665,7 @@ class Server:
             c = ServerClient(server=self, stream=stream)
             self._clients.add(c)
             await c.run()
-        except ClosedResourceError:
+        except (ClosedResourceError, anyio.EndOfStream):
             self.logger.debug("XX %d closed", c._client_nr)
         except BaseException as exc:
             CancelExc = anyio.get_cancelled_exc_class()
@@ -2637,19 +2673,19 @@ class Server:
                 # pylint: disable=no-member
                 exc = exc.filter(lambda e: None if isinstance(e, CancelExc) else e, exc)
             if exc is not None and not isinstance(exc, CancelExc):
-                if isinstance(exc, ClosedResourceError):
+                if isinstance(exc, (ClosedResourceError, anyio.EndOfStream)):
                     self.logger.debug("XX %d closed", c._client_nr)
                 else:
                     self.logger.exception("Client connection killed", exc_info=exc)
             try:
-                async with anyio.move_on_after(2) as cs:
+                with anyio.move_on_after(2) as cs:
                     cs.shield = True
                     if c is not None:
                         await c.send({"error": str(exc)})
             except Exception:
                 pass
         finally:
-            async with anyio.move_on_after(2, shield=True):
+            with anyio.move_on_after(2, shield=True):
                 if c is not None:
                     self._clients.remove(c)
                 await stream.aclose()
