@@ -1,26 +1,27 @@
+import importlib
+import logging
 import os
 import sys
-import asyncclick as click
 from collections import defaultdict
 from collections.abc import Mapping
-from pathlib import Path
-from functools import partial
-import importlib
 from contextvars import ContextVar
+from functools import partial
+from logging.config import dictConfig
+from pathlib import Path
 from typing import Awaitable
 
+import asyncclick as click
+
 from ._dict import attrdict, combine_dict
 from ._impl import NotGiven
-from ._path import path_eval, P
+from ._path import P, path_eval
 from ._yaml import yload
 
-import logging
-from logging.config import dictConfig
-
 logger = logging.getLogger(__name__)
 
 __all__ = [
     "main_",
+    "read_cfg",
     "wrap_main",
     "Loader",
     "load_subgroup",
@@ -37,16 +38,17 @@ NoneType = type(None)
 
 def attr_args(proc):
     """
-    Allow vars_/eval_/path_ args
+    Attach the standard ``-v``/``-e``/``-p`` arguments to a ``click.command``.
+    Passes ``vars_``/``eval_``/``path_`` args.
     """
     proc = click.option(
-        "-v",
-        "--var",
-        "vars_",
+        "-p",
+        "--path",
+        "path_",
         nargs=2,
-        type=(P, str),
+        type=(P, P),
         multiple=True,
-        help="Parameter (name value)",
+        help="Parameter (name value), as path",
     )(proc)
     proc = click.option(
         "-e",
@@ -58,24 +60,24 @@ def attr_args(proc):
         help="Parameter (name value), evaluated",
     )(proc)
     proc = click.option(
-        "-p",
-        "--path",
-        "path_",
+        "-v",
+        "--var",
+        "vars_",
         nargs=2,
-        type=(P, P),
+        type=(P, str),
         multiple=True,
-        help="Parameter (name value), as path",
+        help="Parameter (name value)",
     )(proc)
     return proc
 
 
 def process_args(val, vars_, eval_, path_, vs=None):
     """
-    process vars_+eval_+path_ args.
+    process ``vars_``/``eval_``/``path_`` args.
 
     Arguments:
         vd: dict to modify
-        vars_, eval_, path_: from the function's arguments
+        vars_, eval_, path_: via `attr_args`
         vs: if given: set of vars
     Returns:
         the new value.
@@ -90,7 +92,9 @@ def process_args(val, vars_, eval_, path_, vs=None):
                 v = NotGiven
             elif v == "/":  # pylint: disable=W0631
                 if vs is None:
-                    raise RuntimeError("A slash value doesn't work here.")
+                    raise click.BadOptionUsage(
+                        option_name=k, message="A slash value doesn't work here."
+                    )
                 v = NoneType
             else:
                 v = path_eval(v)  # pylint: disable=W0631
@@ -102,29 +106,59 @@ def process_args(val, vars_, eval_, path_, vs=None):
     for k, v in data():
         if not k:
             if vs is not None:
-                raise RuntimeError("You can't use empty paths here.")
+                raise click.BadOptionUsage(
+                    option_name=k, message="You can't use empty paths here."
+                )
             if n:
-                raise RuntimeError("Setting a single value conflicts.")
+                raise click.BadOptionUsage(
+                    option_name=k, message="Setting a single value conflicts."
+                )
             val = v
             n = -1
         elif n < 0:
-            raise RuntimeError("Setting a single value conflicts.")
+            raise click.BadOptionUsage(option_name=k, message="Setting a single value conflicts.")
         else:
             if not isinstance(val, Mapping):
                 val = attrdict()
             if vs is not None:
-                vs.add(k)
+                vs.add(str(k))
             if v is NotGiven:
                 val = attrdict._delete(val, k)
             elif v is NoneType:
                 val = attrdict._delete(val, k)
-                vs.discard(k)
+                vs.discard(str(k))
             else:
                 val = attrdict._update(val, k, v)
             n += 1
     return val
 
 
+def read_cfg(name, path):
+    cfg = None
+
+    def _cfg(path):
+        nonlocal cfg
+        if cfg is not None:
+            return
+        if os.path.exists(path):
+            try:
+                with open(path, "r") as cf:
+                    cfg = yload(cf)
+            except PermissionError:
+                pass
+
+    if name is not None and cfg is not False:
+        if path is not None:
+            _cfg(path)
+        else:
+            _cfg(os.path.expanduser(f"~/config/{name}.cfg"))
+            _cfg(os.path.expanduser(f"~/.config/{name}.cfg"))
+            _cfg(os.path.expanduser(f"~/.{name}.cfg"))
+            _cfg(f"/etc/{name}/{name}.cfg")
+            _cfg(f"/etc/{name}.cfg")
+        return cfg
+
+
 def load_one(path, name, endpoint=None):
     mod = importlib.import_module(f"{path}.{name}")
     if endpoint is not None:
@@ -318,7 +352,10 @@ class MainLoader(Loader):
 
 
 @load_subgroup(
-    plugin="main", cls=MainLoader, add_help_option=False, invoke_without_command=True
+    plugin="main",
+    cls=MainLoader,
+    add_help_option=False,
+    invoke_without_command=True,
 )  # , __file__, "command"))
 @click.option("-v", "--verbose", count=True, help="Be more verbose. Can be used multiple times.")
 @click.option("-q", "--quiet", count=True, help="Be less verbose. Opposite of '--verbose'.")
@@ -326,7 +363,7 @@ class MainLoader(Loader):
 @click.option(
     "-l", "--log", multiple=True, help="Adjust log level. Example: '--log asyncactor=DEBUG'."
 )
-@click.option("-c", "--cfg", type=click.File("r"), default=None, help="Configuration file (YAML).")
+@click.option("-c", "--cfg", type=click.Path("r"), default=None, help="Configuration file (YAML).")
 @click.option(
     "-C",
     "--conf",
@@ -343,12 +380,13 @@ async def main_(ctx, verbose, quiet, help=False, **kv):  # pylint: disable=redef
 
     You need to add a subcommand for this to do anything.
     """
+    ctx.allow_interspersed_args = True
 
     # The above `MainLoader.invoke` call causes this code to be called
     # twice instead of never.
     if ctx.obj is not None:
         return
-    wrap_main(ctx=ctx, verbose=verbose - quiet, **kv)
+    wrap_main(ctx=ctx, verbose=max(0, 1 + verbose - quiet), **kv)
     if help or ctx.invoked_subcommand is None and not ctx.protected_args:
         print(ctx.get_help())
         ctx.exit()
@@ -431,35 +469,14 @@ def wrap_main(  # pylint: disable=redefined-builtin
             pass
 
     obj.stdout = CFG.get("_stdout", sys.stdout)  # used for testing
-
-    def _cfg(path):
-        nonlocal cfg
-        if cfg is not None:
-            return
-        if os.path.exists(path):
-            try:
-                cfg = open(path, "r")
-            except PermissionError:
-                pass
-
-    if name is not None and cfg is not False:
-        _cfg(os.path.expanduser(f"~/config/{name}.cfg"))
-        _cfg(os.path.expanduser(f"~/.config/{name}.cfg"))
-        _cfg(os.path.expanduser(f"~/.{name}.cfg"))
-        _cfg(f"/etc/{name}/{name}.cfg")
-        _cfg(f"/etc/{name}.cfg")
-
     obj.CFG = CFG
 
+    cfg = read_cfg(name, cfg)
+
     if cfg:
         logger.debug("Loading %s", cfg)
 
-        cd = yload(cfg)
-        if cd is None:
-            obj.cfg = CFG
-        else:
-            obj.cfg = combine_dict(cd, CFG, cls=attrdict)
-        cfg.close()
+        obj.cfg = combine_dict(cfg, CFG, cls=attrdict)
     else:
         obj.cfg = CFG
 
